@@ -1,4 +1,5 @@
-"""The Textual command palette.
+"""
+This module contains classes for working with Textual's command palette.
 
 See the guide on the [Command Palette](../guide/command_palette.md) for full details.
 
@@ -38,7 +39,7 @@ from .message import Message
 from .reactive import var
 from .screen import Screen, SystemModalScreen
 from .timer import Timer
-from .types import CallbackType, IgnoreReturnCallbackType
+from .types import IgnoreReturnCallbackType
 from .widget import Widget
 from .widgets import Button, Input, LoadingIndicator, OptionList, Static
 from .widgets.option_list import Option
@@ -418,9 +419,11 @@ class CommandInput(Input):
     """
 
 
-class CommandPalette(SystemModalScreen[CallbackType]):
+class CommandPalette(SystemModalScreen):
     """The Textual command palette."""
 
+    AUTO_FOCUS = "CommandInput"
+
     COMPONENT_CLASSES: ClassVar[set[str]] = {
         "command-palette--help-text",
         "command-palette--highlight",
@@ -865,7 +868,7 @@ class CommandPalette(SystemModalScreen[CallbackType]):
             clear_current: Should the current content of the list be cleared first?
         """
         # For the moment, this is a fairly naive approach to populating the
-        # command list with a sorted list of commands. Every time we add a
+        # command list with a list of commands. Every time we add a
         # new one we're nuking the list of options and populating them
         # again. If this turns out to not be a great approach, we may try
         # and get a lot smarter with this (ideally OptionList will grow a
@@ -876,7 +879,7 @@ class CommandPalette(SystemModalScreen[CallbackType]):
             if command_list.highlighted is not None and not clear_current
             else None
         )
-        command_list.clear_options().add_options(sorted(commands, reverse=True))
+        command_list.clear_options().add_options(commands)
         if highlighted is not None and highlighted.id:
             command_list.highlighted = command_list.get_option_index(highlighted.id)
         self._list_visible = bool(command_list.option_count)
@@ -1076,7 +1079,8 @@ class CommandPalette(SystemModalScreen[CallbackType]):
                 # decide what to do with it (hopefully it'll run it).
                 self._cancel_gather_commands()
                 self.app.post_message(CommandPalette.Closed(option_selected=True))
-                self.dismiss(self._selected_command.command)
+                self.dismiss()
+                self.call_later(self._selected_command.command)
 
     @on(OptionList.OptionHighlighted)
     def _stop_event_leak(self, event: OptionList.OptionHighlighted) -> None:
