@@ -9,7 +9,7 @@ import rich.repr
 from rich.style import NULL_STYLE, Style
 from rich.text import Text, TextType
 
-from .. import events
+from .. import events, on
 from .._immutable_sequence_view import ImmutableSequenceView
 from .._loop import loop_last
 from .._segment_tools import line_pad
@@ -50,6 +50,10 @@ class UnknownNodeID(Exception):
     """Exception raised when referring to an unknown [`TreeNode`][textual.widgets.tree.TreeNode] ID."""
 
 
+class AddNodeError(Exception):
+    """Exception raised when there is an error with a request to add a node."""
+
+
 @dataclass
 class _TreeLine(Generic[TreeDataType]):
     path: list[TreeNode[TreeDataType]]
@@ -70,13 +74,13 @@ class _TreeLine(Generic[TreeDataType]):
             Width in cells.
         """
         if show_root:
-            return 2 + (max(0, len(self.path) - 1)) * guide_depth
+            width = (max(0, len(self.path) - 1)) * guide_depth
         else:
-            guides = 2
+            width = 0
             if len(self.path) > 1:
-                guides += (len(self.path) - 1) * guide_depth
+                width += (len(self.path) - 1) * guide_depth
 
-        return guides
+        return width
 
 
 class TreeNodes(ImmutableSequenceView["TreeNode[TreeDataType]"]):
@@ -322,6 +326,8 @@ class TreeNode(Generic[TreeDataType]):
         label: TextType,
         data: TreeDataType | None = None,
         *,
+        before: int | TreeNode[TreeDataType] | None = None,
+        after: int | TreeNode[TreeDataType] | None = None,
         expand: bool = False,
         allow_expand: bool = True,
     ) -> TreeNode[TreeDataType]:
@@ -330,34 +336,101 @@ class TreeNode(Generic[TreeDataType]):
         Args:
             label: The new node's label.
             data: Data associated with the new node.
+            before: Optional index or `TreeNode` to add the node before.
+            after: Optional index or `TreeNode` to add the node after.
             expand: Node should be expanded.
             allow_expand: Allow use to expand the node via keyboard or mouse.
 
         Returns:
             A new Tree node
+
+        Raises:
+            AddNodeError: If there is a problem with the addition request.
+
+        Note:
+            Only one of `before` or `after` can be provided. If both are
+            provided a `AddNodeError` will be raised.
         """
+        if before is not None and after is not None:
+            raise AddNodeError("Unable to add a node both before and after a node")
+
+        insert_index: int = len(self.children)
+
+        if before is not None:
+            if isinstance(before, int):
+                insert_index = before
+            elif isinstance(before, TreeNode):
+                try:
+                    insert_index = self.children.index(before)
+                except ValueError:
+                    raise AddNodeError(
+                        "The node specified for `before` is not a child of this node"
+                    )
+            else:
+                raise TypeError(
+                    "`before` argument must be an index or a TreeNode object to add before"
+                )
+
+        if after is not None:
+            if isinstance(after, int):
+                insert_index = after + 1
+                if after < 0:
+                    insert_index += len(self.children)
+            elif isinstance(after, TreeNode):
+                try:
+                    insert_index = self.children.index(after) + 1
+                except ValueError:
+                    raise AddNodeError(
+                        "The node specified for `after` is not a child of this node"
+                    )
+            else:
+                raise TypeError(
+                    "`after` argument must be an index or a TreeNode object to add after"
+                )
+
         text_label = self._tree.process_label(label)
         node = self._tree._add_node(self, text_label, data)
         node._expanded = expand
         node._allow_expand = allow_expand
         self._updates += 1
-        self._children.append(node)
+        self._children.insert(insert_index, node)
         self._tree._invalidate()
         return node
 
     def add_leaf(
-        self, label: TextType, data: TreeDataType | None = None
+        self,
+        label: TextType,
+        data: TreeDataType | None = None,
+        *,
+        before: int | TreeNode[TreeDataType] | None = None,
+        after: int | TreeNode[TreeDataType] | None = None,
     ) -> TreeNode[TreeDataType]:
         """Add a 'leaf' node (a node that can not expand).
 
         Args:
             label: Label for the node.
             data: Optional data.
+            before: Optional index or `TreeNode` to add the node before.
+            after: Optional index or `TreeNode` to add the node after.
 
         Returns:
             New node.
+
+        Raises:
+            AddNodeError: If there is a problem with the addition request.
+
+        Note:
+            Only one of `before` or `after` can be provided. If both are
+            provided a `AddNodeError` will be raised.
         """
-        node = self.add(label, data, expand=False, allow_expand=False)
+        node = self.add(
+            label,
+            data,
+            before=before,
+            after=after,
+            expand=False,
+            allow_expand=False,
+        )
         return node
 
     def _remove_children(self) -> None:
@@ -495,7 +568,7 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
     guide_depth = reactive(4, init=False)
     """The indent depth of tree nodes."""
     auto_expand = var(True)
-    """Auto expand tree nodes when clicked."""
+    """Auto expand tree nodes when they are selected."""
 
     LINES: dict[str, tuple[str, str, str, str]] = {
         "default": (
@@ -745,7 +818,7 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
         self.root.data = data
         return self
 
-    def select_node(self, node: TreeNode[TreeDataType] | None) -> None:
+    def move_cursor(self, node: TreeNode[TreeDataType] | None) -> None:
         """Move the cursor to the given node, or reset cursor.
 
         Args:
@@ -753,6 +826,23 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
         """
         self.cursor_line = -1 if node is None else node._line
 
+    def select_node(self, node: TreeNode[TreeDataType] | None) -> None:
+        """Move the cursor to the given node and select it, or reset cursor.
+
+        Args:
+            node: A tree node to move the cursor to and select, or None to reset cursor.
+        """
+        self.move_cursor(node)
+        if node is not None:
+            self.post_message(Tree.NodeSelected(node))
+
+    @on(NodeSelected)
+    def _expand_node_on_select(self, event: NodeSelected[TreeDataType]) -> None:
+        """When the node is selected, expand the node if `auto_expand` is True."""
+        node = event.node
+        if self.auto_expand:
+            self._toggle_node(node)
+
     def get_node_at_line(self, line_no: int) -> TreeNode[TreeDataType] | None:
         """Get the node for a given line.
 
@@ -822,6 +912,10 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
         else:
             self.hover_line = -1
 
+    def _on_leave(self, _: events.Leave) -> None:
+        # Ensure the hover effect doesn't linger after the mouse leaves.
+        self.hover_line = -1
+
     def _new_id(self) -> NodeID:
         """Create a new node ID.
 
@@ -1233,6 +1327,4 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
             pass
         else:
             node = line.path[-1]
-            if self.auto_expand:
-                self._toggle_node(node)
-            self.post_message(self.NodeSelected(node))
+            self.post_message(Tree.NodeSelected(node))
